/*
 *  chunkfs.h - header file for chunkfs implementation in FUSE
 *
 *  Copyright (C) 2006 Amit Gud <gud@ksu.edu>
 *
 *  This program can be distributed under the terms of the GNU GPL v2.
 *  See the file COPYING.
 */

#ifndef __CHUNKFS_H
#define __CHUNKFS_H

#include <stdlib.h>
#include <assert.h>
#include <sys/stat.h>

#include <ext2fs/ext2fs.h>
#include <ext2fs/ext2_fs.h>

/** common code **/

#define DEBUG

#define CHUNKFS_ROOT_INO EXT2_ROOT_INO

struct dirbuf {
	char *p;
	size_t size;
};

#ifdef DEBUG
#define dbg(fmt, arg...) \
	do { \
		printf("%s: " fmt "\n", __FUNCTION__,  ## arg); \
	} while(0)
#else
#define dbg(fmt, arg...)
#endif

#define min(x, y) ((x) < (y) ? (x) : (y))

/** for chunkfs code **/
typedef struct ext2_inode chunkfs_inode_t;
typedef ext2_ino_t chunkfs_ino_t;
typedef ext2_file_t chunkfs_file_t;
typedef ext2_filsys chunk_filsys;

#define CHUNKFS_INO(ino) ((ext2_ino_t)(ino = (ino) < CHUNKFS_ROOT_INO ? CHUNKFS_ROOT_INO : (ino)))
#define CHUNKFS_FILE(efile) ((void *) (unsigned long) (efile))

/** function declarations **/
/* chunkfs.c */
int chunkfs_dirent_size(int);
char *chunkfs_add_dirent(char *, const char *, const struct stat *, off_t);

/* ext2fs.c */
void fill_statbuf(ext2_ino_t, struct ext2_inode *, struct stat *);

int read_inode(ext2_filsys, ext2_ino_t, struct ext2_inode *);
int write_inode(ext2_filsys, ext2_ino_t, struct ext2_inode *);
int get_ino_by_name(ext2_filsys, ext2_ino_t ,const char *, ext2_ino_t *);

int do_lookup(ext2_filsys, ext2_ino_t, const char *, ext2_ino_t *, struct ext2_inode *);
int do_dir_iterate(ext2_filsys, ext2_ino_t, int, struct dirbuf *);
ext2_file_t do_open(ext2_filsys, ext2_ino_t, int);
int do_create(ext2_filsys, ext2_ino_t, const char *, mode_t,
				ext2_ino_t *, struct ext2_inode *);
void kill_file_by_inode(ext2_filsys, ext2_ino_t);
int do_link(ext2_filsys, ext2_ino_t, const char *, ext2_ino_t, int);
int do_unlink(ext2_filsys, ext2_ino_t, const char *, int);
int do_read(struct ext2_file *, ext2_ino_t, size_t, off_t, char *, unsigned int *);
int do_write(struct ext2_file *, ext2_ino_t, const char *, size_t, off_t, unsigned int *);
int do_file_flush(struct ext2_file *fh);
int do_file_close(struct ext2_file *fh);
int do_mkdir(ext2_filsys, ext2_ino_t, const char *, mode_t, ext2_ino_t *);
int do_rmdir(ext2_filsys, ext2_ino_t, const char *);

#endif /* __CHUNKFS_H */
